@@ -1,45 +1,46 @@
 """
-automatic width for kivy widgets
-================================
+automatic width mix-in classes for kivy widgets
+===============================================
 
-This ae portion is providing mix-in-classes for to automatically size and resize kivy widgets.
+This ae portion is providing classes to mix them into any kivy widget with a `texture` property (like e.g.
+:class:`~kivy.uix.label.Label` or :class:`~kivy.uix.button.Button`), to automatically size and resize widgets and/or to
+display a long text as scrolling ticker within a tall widget.
 
 
 automatic font size iteration with animation
 --------------------------------------------
 
-Mix-in the :class:`AutoFontSizeBehavior` for to grow or shrink the font size automatically to fully fill the
-width/height of a widget with its texture.
+Mix-in the :class:`AutoFontSizeBehavior` into any kivy widget with a `texture` property, to automatically grow or shrink
+the font size to fully fill the width/height of the widget with the size of their texture.
 
 More details see in the documentation of the :class:`AutoFontSizeBehavior` class.
 
 
-automatic ticker animation
---------------------------
-
-For to automatic slide the texture of a widget if it is too big to be completely/fully displayed, like in a news
-ticker, simply mix-in the :class:`SimpleAutoTickerBehavior`.
-
-More details see in the documentation of the :class:`SimpleAutoTickerBehavior` class.
-
-
 automatic container width with opening animation
 ------------------------------------------------
 
-The class :class:`ContainerChildrenAutoWidthBehavior` determines the optimal width of a container widget
-so that the text of any its label and button children widgets is fully visible/displayed.
+The class :class:`ContainerChildrenAutoWidthBehavior` determines the optimal width of a container widget, so that the
+text of any its children widgets is fully visible/displayed.
 
-The optimal container width is determined by increasing width of the container in iterations,
-which are implemented through a kivy :class:`~kivy.animation.Animation`. As soon as the texts of all children
-are fully displayed (or the maximum width is reached) the animation stops.
+The optimal container width is determined by increasing width of the container in iterations, which are implemented
+through a kivy :class:`~kivy.animation.Animation`. As soon as the texts of all children are fully displayed (or the
+maximum width is reached) the animation stops.
 
 More details see in the documentation of the :class:`ContainerChildrenAutoWidthBehavior` class.
 
 
+automatic ticker animation
+--------------------------
+
+Mix-in the :class:`SimpleAutoTickerBehavior` class to automatic slide the texture of a widget if it is too big to be
+completely/fully displayed, like in a news-ping-pong-ticker.
+
+For more details check the documentation of the :class:`SimpleAutoTickerBehavior` class.
 """
-from typing import Callable, Optional
+from typing import Callable, Dict, Optional
 
 from kivy.animation import Animation                                                        # type: ignore
+from kivy.clock import Clock                                                                # type: ignore
 from kivy.core.window import Window                                                         # type: ignore
 from kivy.factory import Factory                                                            # type: ignore
 # pylint: disable=no-name-in-module
@@ -54,16 +55,17 @@ except ImportError:
     MAX_FONT_SIZE = 69
 
 
-__version__ = '0.1.14'
+__version__ = '0.1.15'
 
 
 class AutoFontSizeBehavior:
-    """ mix-in for to interpolate the optimal font size so that the texture is filling the full width of the widget.
+    """ mix-in to interpolate the optimal font size so that the texture is filling the full width of the widget.
 
-    The desired spacing (left plus right) between the texture and the widget borders can be set via the
-    :attr:`auto_font_text_spacing` property.
+    The desired spacing (left plus right) between the texture border and the widget borders can be set via the
+    :attr:`auto_font_text_spacing` property. Additional padding can be added via the `padding_x` property of the
+    mixing-in widget.
 
-    For to disable the animation set the length of the :data:`auto_font_anim_duration` to zero or very short value.
+    To disable the animation set the length of the :data:`auto_font_anim_duration` to zero or very short value.
 
     The minimum and maximum of the texture font size can be restricted by setting the attributes
     :attr:`auto_font_min_size` and :attr:`auto_font_max_size`.
@@ -78,31 +80,29 @@ class AutoFontSizeBehavior:
 
     # public attributes
     auto_font_anim_duration: float = NumericProperty(0.9)
-    """ duration of the font size grow/shrink animation in seconds.
+    """ duration in seconds of the font size grow/shrink animation.
 
-    :attr:`auto_font_anim_duration` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 0.9 seconds.
+    :attr:`auto_font_anim_duration` is a :class:`~kivy.properties.NumericProperty` and defaults to 0.9 seconds.
     """
 
     auto_font_max_size: float = NumericProperty(MAX_FONT_SIZE)
     """ maximum font size.
 
-    :attr:`auto_font_max_size` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to :data:`~ae.gui_app.MAX_FONT_SIZE`.
+    :attr:`auto_font_max_size` is a :class:`~kivy.properties.NumericProperty` and defaults to
+    :data:`~ae.gui_app.MAX_FONT_SIZE`.
     """
 
     auto_font_min_size: float = NumericProperty(MIN_FONT_SIZE)
     """ minimum font size.
 
-    :attr:`auto_font_min_size` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to :data:`~ae.gui_app.MIN_FONT_SIZE`.
+    :attr:`auto_font_min_size` is a :class:`~kivy.properties.NumericProperty` and defaults to
+    :data:`~ae.gui_app.MIN_FONT_SIZE`.
     """
 
     auto_font_text_spacing: float = NumericProperty('18sp')
-    """ horizontal padding between widget and texture width in pixels.
+    """ horizontal padding in pixels between widget and texture width (including the additional horizontal `padding_x`).
 
-    :attr:`auto_font_text_spacing` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 18sp.
+    :attr:`auto_font_text_spacing` is a :class:`~kivy.properties.NumericProperty` and defaults to 18sp.
     """
 
     # internal attributes
@@ -169,17 +169,16 @@ Factory.register('AutoFontSizeBehavior', cls=AutoFontSizeBehavior)
 class ContainerChildrenAutoWidthBehavior:
     """ detect minimum width for the complete display of the textures of all children at opening with animation.
 
-    This mix-in class can be added to any type of container or layout widget for to provide a consistent API
-    with :meth:`.open` and :meth:`.close` methods, a :meth:`.on_complete_opened` event and a :attr:`.container`
-    attribute.
+    This mix-in class can be added to any type of container or layout widget to provide a consistent API with
+    :meth:`.open` and :meth:`.close` methods, a :meth:`.on_complete_opened` event and a :attr:`.container` attribute.
 
-    A `container` attribute will be automatically created for container classes without it.
+    .. note:: A `container` attribute will be automatically created for container classes without it.
 
-    The animation starts when the :meth:`~ContainerChildrenAutoWidthBehavior.open` method get called.
-    This call will be forwarded via `super()` to the container if it has an `open` method.
+    The animation starts when the :meth:`~ContainerChildrenAutoWidthBehavior.open` method get called. This call will be
+    forwarded via `super()` to the container if it has an `open` method.
 
-    At animation start the width of this container will be set to value of the :attr:`auto_width_start`
-    attribute. Then the container width increases via the running animation until, either:
+    At animation start the width of this container will be set to value of the :attr:`auto_width_start` attribute. Then
+    the container width increases via the running animation until, either:
 
     * the container width is greater than the value of the :attr:`auto_width_minimum` attribute
       and the textures of all children are fully visible or
@@ -187,8 +186,7 @@ class ContainerChildrenAutoWidthBehavior:
     * the container width reaches the app window width minus the window padding specified in the
       :attr:`auto_width_window_padding` attribute.
 
-    The window width gets bound to the container width for to ensure proper displaying if
-    the window width changes.
+    The window width gets bound to the container width to ensure proper displaying if the window width changes.
 
     :Events:
         `on_complete_opened`:
@@ -203,54 +201,46 @@ class ContainerChildrenAutoWidthBehavior:
     width: float                #: width of the widget to be mixed into (mostly a parent of self.container)
 
     auto_width_anim_duration: float = NumericProperty(0.9)
-    """ duration of the width animation in seconds.
+    """ duration in seconds of the auto-width-animation.
 
-    :attr:`auto_width_anim_duration` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 0.9 seconds.
+    :attr:`auto_width_anim_duration` is a :class:`~kivy.properties.NumericProperty` and defaults to 0.9 seconds.
     """
 
     auto_width_window_padding: float = NumericProperty('96sp')
-    """ horizontal padding between the window and the container in pixels.
+    """ horizontal padding in pixels between the window and the container.
 
-    :attr:`auto_width_window_padding` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 96sp.
+    :attr:`auto_width_window_padding` is a :class:`~kivy.properties.NumericProperty` and defaults to 96sp.
     """
 
     auto_width_minimum: float = NumericProperty('369sp')
     """ minimum container width in pixels (before the width animation will be stopped).
 
-    :attr:`auto_width_minimum` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 369sp.
+    :attr:`auto_width_minimum` is a :class:`~kivy.properties.NumericProperty` and defaults to 369sp.
     """
 
     auto_width_child_padding: float = NumericProperty('87sp')
-    """ horizontal padding between child widget and child texture in pixels.
+    """ horizontal padding in pixels between child widget and child texture.
 
-    :attr:`auto_width_child_padding` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 87sp.
+    :attr:`auto_width_child_padding` is a :class:`~kivy.properties.NumericProperty` and defaults to 87sp.
     """
 
     auto_width_start: float = NumericProperty('3sp')
     """ container width in pixels at the start of the width animation.
 
-    :attr:`auto_width_start` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 3sp.
+    :attr:`auto_width_start` is a :class:`~kivy.properties.NumericProperty` and defaults to 3sp.
     """
 
     # internal attributes
     _width_anim: Animation = None
     _complete_width: float = 0.0
 
-    def __init__(self, **kwargs):
-        # noinspection PyUnresolvedReferences
-        self.register_event_type('on_complete_opened')  # pylint: disable=maybe-no-member
-        super().__init__(**kwargs)
+    __events__ = ('on_complete_opened', )
 
     def close(self, *_args, **_kwargs):
         """ close/dismiss container/layout (ae.gui_app popup handling compatibility for all GUI frameworks).
 
-        :param _args:           unused argument (for to have compatible signature for DropDown/Popup/ModalView widgets).
-        :param _kwargs:         unused argument (for to have compatible signature for DropDown/Popup/ModalView widgets).
+        :param _args:           unused argument (to have compatible signature for DropDown/Popup/ModalView widgets).
+        :param _kwargs:         unused argument (to have compatible signature for DropDown/Popup/ModalView widgets).
         """
         try:
             # noinspection PyUnresolvedReferences
@@ -263,12 +253,12 @@ class ContainerChildrenAutoWidthBehavior:
                     self.parent.remove_widget(self)
 
     def on_complete_opened(self):
-        """ dispatch event when opening and width got determined. """
+        """ dispatch event default handler, called on opening when the final width got determined. """
 
     def open(self, *_args, **kwargs):
-        """ start optional open animation after calling open method if exists in inheriting container/layout widget.
+        """ open container, optionally starting auto-width-animation.
 
-        :param _args:           unused argument (for to have compatible signature for Popup/ModalView and DropDown
+        :param _args:           unused argument (to have compatible signature for Popup/ModalView and DropDown
                                 widgets passing the parent widget).
         :param kwargs:          extra arguments that are removed before to be passed to the inheriting open method:
 
@@ -279,7 +269,7 @@ class ContainerChildrenAutoWidthBehavior:
         if not hasattr(super(), 'container'):
             self.container = getattr(self, '_container', self)  # Popup has _container attribute, BoxLayout=self
 
-        if callable(getattr(super(), 'open', None)):
+        if callable(getattr(super(), 'open', None)):    # call open method if exists in inheriting container/layout
             # noinspection PyUnresolvedReferences
             super().open(*_args, **kwargs)      # pylint: disable=maybe-no-member
 
@@ -337,20 +327,18 @@ Factory.register('ContainerChildrenAutoWidthBehavior', cls=ContainerChildrenAuto
 
 
 class SimpleAutoTickerBehavior:
-    """ mix-in class for to slide texture if too long to be displayed completely.
+    """ mix-in class to slide texture in a ping-pong-like-ticker animation, if too long to be displayed completely.
 
-    If the `text` or `size` of the widget where this class get mixed in changes then this instance is first
-    determining the number of characters that can be displayed completely in this widget. This is done with
-    an kivy animation. The duration of this animation can be set via the property
-    :attr:`auto_ticker_length_anim_duration`.
+    If the `text` or `size` of the widget where this class get mixed in changes then this instance is first determining
+    the number of characters that can be displayed completely in this widget. This is done with an kivy animation. The
+    duration of this animation can be set via the property :attr:`auto_ticker_length_anim_duration`.
 
-    For to adjust the padding space between the widget border and their texture width, the property
-    :attr:`auto_ticker_text_spacing` can be changed accordingly.
+    To adjust the padding space between the widget border and their texture width, the property
+    :attr:`auto_ticker_text_spacing` can be set accordingly.
 
-    After determining the maximum number of characters that can be displayed and storing this value into
-    the private attribute :attr:`_ticker_text_length` a second animation gets started for to slide/scroll
-    the text. The duration of the second animation can be set via the property
-    :attr:`auto_ticker_offset_anim_duration`.
+    After determining the maximum number of characters that can be displayed and storing this value into the private
+    attribute :attr:`_ticker_text_length` a second animation - the offset animation - gets started to slide/scroll the
+    text. The speed of the offset animation can be set via the property :attr:`auto_ticker_offset_anim_speed`.
 
     .. note::
         While the ticker animations are running the `text` property of the widget is only containing the
@@ -360,114 +348,126 @@ class SimpleAutoTickerBehavior:
     """
     # abstracts
     bind: Callable
+    get_root_window: Callable
     text: str
     texture_size: tuple
     texture_update: Callable
+    unbind: Callable
     width: float
 
     # public attributes
     auto_ticker_length_anim_duration: float = NumericProperty(0.9)
-    """ duration in seconds of the iteration animation for to determine the maximum text length.
+    """ duration in seconds of the iteration animation to determine the maximum text length.
 
-    :attr:`auto_ticker_length_anim_duration` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 0.9 seconds.
+    :attr:`auto_ticker_length_anim_duration` is a :class:`~kivy.properties.NumericProperty` and defaults to 0.9 seconds.
     """
 
-    auto_ticker_offset_anim_duration: float = NumericProperty(9.6)
-    """ duration of the font size grow/shrink animation in seconds.
+    auto_ticker_offset_anim_speed: float = NumericProperty(9.6)
+    """ speed of the ticker text offset animation in characters per second.
 
-    :attr:`auto_ticker_length_anim_duration` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 9.6 seconds.
+    :attr:`auto_ticker_length_anim_duration` is a :class:`~kivy.properties.NumericProperty` and defaults to 9.6.
     """
 
     auto_ticker_text_spacing: float = NumericProperty('18sp')
     """ horizontal padding between widget and texture width in pixels.
 
-    :attr:`auto_ticker_text_spacing` is a :class:`~kivy.properties.NumericProperty` and
-    defaults to 18sp.
+    :attr:`auto_ticker_text_spacing` is a :class:`~kivy.properties.NumericProperty` and defaults to 18sp.
     """
 
     # internal attributes
-    _length_anim: Optional[Animation] = None
-    _offset_anim: Optional[Animation] = None
-    _ori_text: str = ""
+    _bound_properties: Dict[str, Callable]      #: properties of the mixing in widget to bind
+    _length_anim: Optional[Animation] = None    #: shorten length animation
+    _min_text_len: int = 6                      #: minimal length of shortened text
+    _offset_anim: Optional[Animation] = None    #: ticker text offset animation
+    _ori_text: str = ""                         #: original/full text string
     _ticker_text_offset: int = 0                #: current animated offset in the ticker text
-    _ticker_text_length: int = 1                #: number of characters that are completely visible in widget
-    _ticker_texture_width: float = 0.0          #: texture width of the complete text
+    _ticker_text_length: int = _min_text_len    #: number of characters that are completely visible in widget
     _ticker_text_updating: bool = False         #: flag to block restart of ticker on internal update of `text` property
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.bind(height=self._start_length_anim)
-        self.bind(text=self._text_changed)
-        self.bind(width=self._start_length_anim)
+        self._bind_properties()
 
-    def _text_changed(self, *_args):
-        """ called on change of label text. """
-        # assert _args[1] == self.text
-        if not self._ticker_text_updating:
-            self._ori_text = self.text
-            self._start_length_anim()
+    def _bind_properties(self):
+        self._bound_properties = dict(
+            height=self._start_length_anim,
+            text=self._text_changed,
+            width=self._start_length_anim,
+        )
+        self.bind(**self._bound_properties)
 
     def _start_length_anim(self, *_args):
         self.text = self._ori_text
         self._stop_offset_anim()
         self._stop_length_anim()
 
-        self._ticker_texture_width = self.width - self.auto_ticker_text_spacing
-
-        self._length_anim = Animation(_ticker_text_length=9, d=self.auto_ticker_length_anim_duration, t='out_quad')
+        self._length_anim = Animation(_ticker_text_length=self._min_text_len,
+                                      d=self.auto_ticker_length_anim_duration, t='out_quad')
         self._length_anim.bind(on_progress=self._ticker_length_progress)
         self._length_anim.start(self)
 
-    def _start_offset_anim(self, *_args):
-        self._offset_anim = Animation(_ticker_text_offset=0, d=self.auto_ticker_offset_anim_duration)
+    def _start_offset_anim(self, offset_on_complete: int = 0):
+        self._offset_anim = Animation(
+            _ticker_text_offset=offset_on_complete,
+            d=(len(self._ori_text) - self._ticker_text_length) / self.auto_ticker_offset_anim_speed)
         self._offset_anim.bind(on_progress=self._ticker_offset_progress)
         self._offset_anim.start(self)
 
-    def _stop_length_anim(self):
+    def _stop_length_anim(self, reset=True):
         if self._length_anim:
             self._length_anim.stop(self)
             self._length_anim = None
-        self._ticker_text_length = len(self._ori_text)
-        self._ticker_texture_width = 0.0
+        if reset:
+            self._ticker_text_length = len(self._ori_text)
 
-    def _stop_offset_anim(self):
+    def _stop_offset_anim(self, reset=True):
         if self._offset_anim:
             self._offset_anim.stop(self)
             self._offset_anim = None
-        self._ticker_text_offset = 0
+        if reset:
+            self._ticker_text_offset = 0
 
-    def _ticker_length_progress(self, _anim: Animation, _self: Widget, _progress: float):
-        if self.texture_size[0] < self._ticker_texture_width:
+    def _text_changed(self, *_args):
+        """ called on change of label text. assert _args[1] == self.text """
+        if not self._ticker_text_updating:
+            self._ori_text = self.text
+            self._start_length_anim()
+
+    def _ticker_length_progress(self, _anim: Animation, _self: Widget, progress: float):
+        if self.texture_size[0] < self.width - self.auto_ticker_text_spacing:
             self._ticker_text_length = len(self.text)
             self._ticker_text_offset = round(self._ticker_max_offset() / 2)
-            if self._length_anim:
-                self._length_anim.stop(self)
-                self._length_anim = None
+            self._stop_length_anim(reset=False)
             self._start_offset_anim()
         else:
-            self._ticker_text_update(self.text[1:-1])
+            cut_off = int(progress * len(self._ori_text) / 2) + 3
+            self._ticker_text_update(self._ori_text[cut_off:-cut_off])
 
-    def _ticker_max_offset(self):
+    def _ticker_max_offset(self) -> int:
         return round(len(self._ori_text) - self._ticker_text_length)
 
     def _ticker_offset_progress(self, _anim: Animation, _self: Widget, progress: float):
-        beg = int(self._ticker_text_offset)
-        end = beg + int(self._ticker_text_length)
-        self._ticker_text_update(self._ori_text[beg:end])
-        if progress >= 1.0 and self._offset_anim:      # added `and self._offset_anim` for mypy
-            self._offset_anim.stop(self)
-            self._offset_anim = Animation(_ticker_text_offset=0 if beg else self._ticker_max_offset() + 1,
-                                          d=self.auto_ticker_offset_anim_duration)
-            self._offset_anim.bind(on_progress=self._ticker_offset_progress)
-            self._offset_anim.start(self)
+        beg = self._ticker_text_offset
+        end = beg + self._ticker_text_length
+        self._ticker_text_update(self._ori_text[int(beg):int(end)])
+        if progress >= 1.0 and self._offset_anim:  # added `and self._offset_anim` for mypy
+            self._stop_offset_anim(reset=False)
+            if self.get_root_window():
+                Clock.schedule_once(                # pragma: no cover
+                    lambda _dt: self._start_offset_anim(offset_on_complete=0 if beg else self._ticker_max_offset()), 3)
+            else:
+                self._unbind_properties()
 
     def _ticker_text_update(self, text: str):
-        self._ticker_text_updating = True
-        self.text = text
-        self.texture_update()
-        self._ticker_text_updating = False
+        if text != self.text:
+            self._ticker_text_updating = True
+            self.text = text
+            self.texture_update()
+            self._ticker_text_updating = False
+
+    def _unbind_properties(self):
+        if self._bound_properties:
+            self.unbind(**self._bound_properties)
 
 
 Factory.register('SimpleAutoTickerBehavior', cls=SimpleAutoTickerBehavior)
